@@ -1,5 +1,5 @@
 /****************************************************************************
- * RRDtool 1.5.5, Copyright 1997-2015 by Tobi Oetiker
+ * RRDtool 1.5.6, Copyright 1997-2016 by Tobi Oetiker
  ****************************************************************************
  * rrd_rpncalc.c  RPN calculator functions
  ****************************************************************************/
@@ -333,7 +333,10 @@ rpnp_t   *rpn_parse(
 
     rpnp = NULL;
     expr = (char *) expr_const;
-
+    if (! *expr){
+        rrd_set_error("can not parse an empty rpn expression");
+        return NULL;
+    }
     while (*expr) {
         if ((rpnp = (rpnp_t *) rrd_realloc(rpnp, (++steps + 2) *
                                            sizeof(rpnp_t))) == NULL) {
