@@ -1,5 +1,5 @@
 /****************************************************************************
- * RRDtool 1.5.5, Copyright 1997-2015 by Tobi Oetiker
+ * RRDtool 1.5.6, Copyright 1997-2016 by Tobi Oetiker
  ****************************************************************************
  * rrd_rpncalc.h  RPN calculator functions
  ****************************************************************************/
@@ -48,7 +48,7 @@ typedef struct rpn_cdefds_t {
 } rpn_cdefds_t;
 
 #define MAX_VNAME_LEN 255
-#define DEF_NAM_FMT "%255[_A-Za-z0-9]"
+#define DEF_NAM_FMT "%255[_A-Za-z0-9-]"
 
 /* limit imposed by sizeof(rpn_cdefs_t) and rrd.ds_def.par */
 #define DS_CDEF_MAX_RPN_NODES (int)(sizeof(unival)*10 / sizeof(rpn_cdefds_t))
